"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.storeToken = storeToken;
exports.retrieveToken = retrieveToken;
exports.removeToken = removeToken;
exports.checkTokenFreshness = checkTokenFreshness;
exports.shouldRequestNewToken = shouldRequestNewToken;
exports.createApiAction = createApiAction;
exports.actionAsPromise = actionAsPromise;
exports.createTokenApiMiddleware = createTokenApiMiddleware;
exports.TokenApiService = exports.TOKEN_STORAGE_KEY = exports.CALL_TOKEN_API = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _lodash = _interopRequireDefault(require("lodash.startswith"));

var _lodash2 = _interopRequireDefault(require("lodash.map"));

var _lodash3 = _interopRequireDefault(require("lodash.isundefined"));

var _lodash4 = _interopRequireDefault(require("lodash.isfunction"));

var _lodash5 = _interopRequireDefault(require("lodash.isarraylikeobject"));

var _lodash6 = _interopRequireDefault(require("lodash.omitby"));

var _jwtDecode = _interopRequireDefault(require("jwt-decode"));

var _moment = _interopRequireDefault(require("moment"));

var _crossFetch = _interopRequireDefault(require("cross-fetch"));

var CALL_TOKEN_API = '@@CALL_TOKEN_API';
exports.CALL_TOKEN_API = CALL_TOKEN_API;
var TOKEN_STORAGE_KEY = 'reduxMiddlewareAuthToken';
exports.TOKEN_STORAGE_KEY = TOKEN_STORAGE_KEY;
var MIN_TOKEN_LIFESPAN = 300;

var NotImplemented = function NotImplemented(message) {
  this.name = 'NotImplemented';
  this.message = message || 'Method not implemented';
  this.stack = new Error().stack;
};

NotImplemented.prototype = Object.create(Error.prototype);
NotImplemented.prototype.constructor = NotImplemented;

function checkResponseIsOk(response) {
  return response.ok ? response : response.text().then(function (text) {
    return Promise.reject(text);
  });
}

function responseToCompletion(_x) {
  return _responseToCompletion.apply(this, arguments);
}

function _responseToCompletion() {
  _responseToCompletion = (0, _asyncToGenerator2.default)(
  /*#__PURE__*/
  _regenerator.default.mark(function _callee4(response) {
    var contentType, text;
    return _regenerator.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            contentType = response.headers.get('Content-Type');

            if (!(contentType && (0, _lodash.default)(contentType, 'application/json'))) {
              _context4.next = 12;
              break;
            }

            _context4.next = 4;
            return response.text();

          case 4:
            text = _context4.sent;
            _context4.prev = 5;
            return _context4.abrupt("return", JSON.parse(text));

          case 9:
            _context4.prev = 9;
            _context4.t0 = _context4["catch"](5);
            return _context4.abrupt("return", text);

          case 12:
            return _context4.abrupt("return", response.text());

          case 13:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4, this, [[5, 9]]);
  }));
  return _responseToCompletion.apply(this, arguments);
}

function createAsyncAction(type, step, payload) {
  var meta = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var action = {
    type: "".concat(type, "_").concat(step),
    payload: payload,
    meta: Object.assign(meta, {
      asyncStep: step
    })
  };

  if (payload && payload instanceof Error) {
    Object.assign(action.meta, {
      error: true
    });
  }

  return action;
}

function createStartAction(type, payload) {
  return createAsyncAction(type, 'START', payload);
}

function createCompletionAction(type, payload, meta) {
  return createAsyncAction(type, 'COMPLETED', payload, meta);
}

function createFailureAction(type, error) {
  return createAsyncAction(type, 'FAILED', new TypeError(error));
}

function storeToken(key, response) {
  var token = response.token;
  localStorage.setItem(key, JSON.stringify(token));
  return token;
}

function retrieveToken(key) {
  var storedValue = localStorage.getItem(key);

  if (!storedValue) {
    return null;
  }

  try {
    return JSON.parse(storedValue);
  } catch (e) {
    if (e instanceof SyntaxError) {
      return null;
    }

    throw e;
  }
}

function removeToken(key) {
  localStorage.removeItem(key);
}

function checkTokenFreshness(token) {
  var tokenPayload = (0, _jwtDecode.default)(token);

  var expiry = _moment.default.unix(tokenPayload.exp);

  return expiry.diff((0, _moment.default)(), 'seconds') > MIN_TOKEN_LIFESPAN;
}

function shouldRequestNewToken() {
  if (!this.refreshToken) {
    return false;
  }

  var token = retrieveToken();
  return token ? checkTokenFreshness(token) : false;
}

var TokenApiService =
/*#__PURE__*/
function () {
  function TokenApiService(apiAction, dispatch) {
    var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    (0, _classCallCheck2.default)(this, TokenApiService);
    this.apiAction = apiAction;
    this.meta = this.apiAction.meta || {};
    this.dispatch = dispatch;
    this.config = config;
    this.defaultHeaders = this.config.defaultHeaders || {
      'Content-Type': 'application/json'
    };
    this.retrieveRefreshToken = this.configOrDefault('retrieveRefreshToken'); // config or default values

    this.checkTokenFreshness = this.configOrDefault('checkTokenFreshness');
    this.retrieveToken = this.configOrDefault('retrieveToken');
    this.storeToken = this.configOrDefault('storeToken');
    this.addTokenToRequest = this.configOrDefault('addTokenToRequest');
    this.refreshAction = this.configOrDefault('refreshAction');
    this.checkResponseIsOk = this.configOrDefault('checkResponseIsOk');
    this.tokenStorageKey = this.config.tokenStorageKey || TOKEN_STORAGE_KEY;
    this.minTokenLifespan = this.config.minTokenLifespan || MIN_TOKEN_LIFESPAN;
    this.actionKey = this.config.actionKey || CALL_TOKEN_API;
    this.preProcessRequest = this.config.preProcessRequest;
    this.refreshToken = this.config.refreshToken || false;
    this.shouldRequestNewToken = this.configOrDefault('shouldRequestNewToken'); // bind where needed

    this.storeToken = this.storeToken.bind(this, this.tokenStorageKey);
    this.retrieveToken = this.retrieveToken.bind(this, this.tokenStorageKey);
  }

  (0, _createClass2.default)(TokenApiService, [{
    key: "configOrDefault",
    value: function configOrDefault(key) {
      return this.config[key] || this.defaultMethods[key];
    }
  }, {
    key: "configOrNotImplemented",
    value: function configOrNotImplemented(key) {
      var method = this.config[key];

      if (!method) {
        throw new NotImplemented("Please provide ".concat(key, " in config"));
      }

      return method;
    }
  }, {
    key: "completeApiRequest",
    value: function completeApiRequest(type, finalResponse) {
      this.dispatch(createCompletionAction(type, finalResponse, this.meta));
      return finalResponse;
    }
  }, {
    key: "defaultCatchApiRequestError",
    value: function defaultCatchApiRequestError(type, error) {
      return error;
    }
  }, {
    key: "catchApiRequestError",
    value: function catchApiRequestError(type, error) {
      var fn = this.configOrDefault('catchApiRequestError');
      this.dispatch(createFailureAction(type, error));
      return fn(type, error);
    }
  }, {
    key: "preserveHeaderValues",
    value: function preserveHeaderValues(meta, response) {
      var headersToPreserve = meta.preserveHeaders;

      if (Array.isArray(headersToPreserve)) {
        var responseHeaders = response.headers;
        var preservedHeaders = headersToPreserve.reduce(function (headers, header) {
          headers[header] = responseHeaders.get(header);
          return headers;
        }, {});
        meta.preservedHeaders = preservedHeaders;
      }

      return response;
    }
  }, {
    key: "apiRequest",
    value: function apiRequest(fetchArgs, action) {
      var meta = action.meta || {};
      var completeApiRequest = this.completeApiRequest.bind(this, action.type);
      var catchApiRequestError = this.catchApiRequestError.bind(this, action.type);
      var preserveHeaderValues = this.preserveHeaderValues.bind(this, this.meta);
      return _crossFetch.default.apply(this, fetchArgs).then(this.checkResponseIsOk).then(preserveHeaderValues).then(responseToCompletion).then(meta.responseHandler).then(completeApiRequest).catch(catchApiRequestError);
    }
  }, {
    key: "apiRequestPromise",
    value: function apiRequestPromise(fetchArgs) {
      return function () {
        return _crossFetch.default.apply(null, fetchArgs).then(checkResponseIsOk).then(responseToCompletion);
      };
    }
  }, {
    key: "apiCallFromAction",
    value: function apiCallFromAction(action) {
      var token = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      var apiFetchArgs = this.getApiFetchArgsFromActionPayload(action.payload, token);
      this.dispatch(createStartAction(action.type, action.payload));
      return this.apiRequest(apiFetchArgs, action, this.store);
    }
  }, {
    key: "multipleApiCallsFromAction",
    value: function multipleApiCallsFromAction(action) {
      var _this = this;

      var token = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      var meta = action.meta || {};
      var promises = (0, _lodash2.default)(action.payload, function (apiAction) {
        var apiFetchArgs = _this.getApiFetchArgsFromActionPayload(apiAction, token);

        return _this.apiRequestPromise(apiFetchArgs)();
      });
      var completeApiRequest = this.completeApiRequest.bind(this, action.type);
      var catchApiRequestError = this.catchApiRequestError.bind(this, action.type);
      this.dispatch(createStartAction(action.type, action.payload));
      return Promise.all(promises).then(meta.responseHandler).then(completeApiRequest).catch(catchApiRequestError);
    }
  }, {
    key: "getToken",
    value: function () {
      var _getToken = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee() {
        var token;
        return _regenerator.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                token = this.retrieveToken();

                if (!(token instanceof Promise)) {
                  _context.next = 5;
                  break;
                }

                _context.next = 4;
                return token;

              case 4:
                return _context.abrupt("return", _context.sent);

              case 5:
                return _context.abrupt("return", token);

              case 6:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function getToken() {
        return _getToken.apply(this, arguments);
      }

      return getToken;
    }()
  }, {
    key: "getRefreshToken",
    value: function () {
      var _getRefreshToken = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee2() {
        var token;
        return _regenerator.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                token = this.retrieveRefreshToken();

                if (!(token instanceof Promise)) {
                  _context2.next = 5;
                  break;
                }

                _context2.next = 4;
                return token;

              case 4:
                return _context2.abrupt("return", _context2.sent);

              case 5:
                return _context2.abrupt("return", token);

              case 6:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function getRefreshToken() {
        return _getRefreshToken.apply(this, arguments);
      }

      return getRefreshToken;
    }()
  }, {
    key: "defaultAddTokenToRequest",
    value: function defaultAddTokenToRequest(headers, endpoint, body, token) {
      return {
        headers: Object.assign({
          Authorization: "JWT ".concat(token)
        }, headers),
        endpoint: endpoint,
        body: body
      };
    }
  }, {
    key: "getApiFetchArgsFromActionPayload",
    value: function getApiFetchArgsFromActionPayload(payload) {
      var token = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      var authenticate = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
      var headers = payload.headers,
          endpoint = payload.endpoint,
          method = payload.method,
          body = payload.body,
          credentials = payload.credentials;

      if ((0, _lodash3.default)(method)) {
        method = 'GET';
      }

      headers = Object.assign(this.defaultHeaders, headers);

      if (token && authenticate) {
        var _this$addTokenToReque = this.addTokenToRequest(headers, endpoint, body, token);

        headers = _this$addTokenToReque.headers;
        endpoint = _this$addTokenToReque.endpoint;
        body = _this$addTokenToReque.body;
      }

      if ((0, _lodash4.default)(this.preProcessRequest)) {
        var _this$preProcessReque = this.preProcessRequest(headers, endpoint, body);

        headers = _this$preProcessReque.headers;
        endpoint = _this$preProcessReque.endpoint;
        body = _this$preProcessReque.body;
      }

      return [endpoint, (0, _lodash6.default)({
        method: method,
        body: body,
        credentials: credentials,
        headers: headers
      }, _lodash3.default)];
    }
  }, {
    key: "call",
    value: function () {
      var _call = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee3() {
        var _this2 = this;

        var token, refreshToken, refreshAction, refreshApiAction, refreshApiActionMeta, refreshArgs;
        return _regenerator.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return this.getToken();

              case 2:
                token = _context3.sent;
                _context3.next = 5;
                return this.shouldRequestNewToken();

              case 5:
                if (!_context3.sent) {
                  _context3.next = 16;
                  break;
                }

                _context3.next = 8;
                return this.getRefreshToken();

              case 8:
                refreshToken = _context3.sent;
                refreshAction = this.refreshAction(refreshToken);
                refreshApiAction = refreshAction[CALL_TOKEN_API];
                refreshApiActionMeta = refreshApiAction.meta || {};
                refreshArgs = this.getApiFetchArgsFromActionPayload(refreshApiAction.payload, token, refreshApiActionMeta.authenticate);
                return _context3.abrupt("return", _crossFetch.default.apply(null, refreshArgs).then(this.checkResponseIsOk).then(responseToCompletion).then(refreshApiActionMeta.responseHandler).then(this.curriedApiCallMethod).catch(function (error) {
                  _this2.dispatch(createFailureAction(_this2.apiAction.type, error));
                }));

              case 16:
                return _context3.abrupt("return", this.curriedApiCallMethod(token));

              case 17:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function call() {
        return _call.apply(this, arguments);
      }

      return call;
    }()
  }, {
    key: "defaultMethods",
    get: function get() {
      return {
        checkTokenFreshness: checkTokenFreshness,
        retrieveToken: retrieveToken,
        storeToken: storeToken,
        refreshAction: function refreshAction() {},
        retrieveRefreshToken: function retrieveRefreshToken() {},
        shouldRequestNewToken: shouldRequestNewToken,
        addTokenToRequest: this.defaultAddTokenToRequest,
        catchApiRequestError: this.defaultCatchApiRequestError,
        checkResponseIsOk: checkResponseIsOk
      };
    }
  }, {
    key: "apiCallMethod",
    get: function get() {
      return (0, _lodash5.default)(this.apiAction.payload) ? this.multipleApiCallsFromAction : this.apiCallFromAction;
    }
  }, {
    key: "curriedApiCallMethod",
    get: function get() {
      return this.apiCallMethod.bind(this, this.apiAction);
    }
  }]);
  return TokenApiService;
}();

exports.TokenApiService = TokenApiService;

function createApiAction(action) {
  return (0, _defineProperty2.default)({}, CALL_TOKEN_API, action);
}

function actionAsPromise(action, dispatch, config) {
  var actionKey = config.actionKey || CALL_TOKEN_API;
  var apiAction = action()[actionKey];

  if (apiAction) {
    var tokenApiService = new TokenApiService(apiAction, dispatch, config);
    return tokenApiService.call();
  } else {
    return Promise.reject('not an API action!');
  }
}

function createTokenApiMiddleware() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return function (store) {
    return function (next) {
      return function (action) {
        var apiAction = action[config.actionKey];

        if (apiAction === undefined) {
          if (action[CALL_TOKEN_API] === undefined) {
            return next(action);
          }

          apiAction = action[CALL_TOKEN_API];
        }

        var tokenApiService = new TokenApiService(apiAction, store.dispatch, config);
        return tokenApiService.call();
      };
    };
  };
}